package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.Digest;
import cn.com.infosec.crypto.InvalidCipherTextException;
import cn.com.infosec.crypto.Wrapper;
import cn.com.infosec.crypto.digests.SHA1Digest;
import cn.com.infosec.crypto.modes.CBCBlockCipher;
import cn.com.infosec.crypto.params.KeyParameter;
import cn.com.infosec.crypto.params.ParametersWithIV;
import cn.com.infosec.crypto.params.ParametersWithRandom;
import java.security.SecureRandom;

public class DESedeWrapEngine implements Wrapper {
   private static final byte[] IV2 = new byte[]{74, -35, -94, 44, 121, -24, 33, 5};
   private CBCBlockCipher engine;
   private KeyParameter param;
   private ParametersWithIV paramPlusIV;
   private byte[] iv;
   private boolean forWrapping;
   Digest sha1 = new SHA1Digest();
   byte[] digest = new byte[20];

   private byte[] calculateCMSKeyChecksum(byte[] key) {
      byte[] var2;
      byte[] var10000 = var2 = new byte[8];
      this.sha1.update(key, 0, key.length);
      this.sha1.doFinal(this.digest, 0);
      System.arraycopy(this.digest, 0, var2, 0, 8);
      return var10000;
   }

   private boolean checkCMSKeyChecksum(byte[] key, byte[] checksum) {
      byte[] var3 = this.calculateCMSKeyChecksum(key);
      if (checksum.length != var3.length) {
         return false;
      } else {
         for(int key1 = 0; key1 != checksum.length; ++key1) {
            if (checksum[key1] != var3[key1]) {
               return false;
            }
         }

         return true;
      }
   }

   public void init(boolean forWrapping, CipherParameters param) {
      this.forWrapping = forWrapping;
      this.engine = new CBCBlockCipher(new DESedeEngine());
      SecureRandom forWrapping1;
      if (param instanceof ParametersWithRandom) {
         ParametersWithRandom var10000 = (ParametersWithRandom)param;
         param = ((ParametersWithRandom)param).getParameters();
         forWrapping1 = var10000.getRandom();
      } else {
         forWrapping1 = new SecureRandom.<init>();
      }

      ParametersWithIV forWrapping3;
      if (param instanceof KeyParameter) {
         this.param = (KeyParameter)param;
         if (this.forWrapping) {
            DESedeWrapEngine var8 = this;
            SecureRandom var10001 = forWrapping1;
            byte[] forWrapping2;
            this.iv = forWrapping2 = new byte[8];
            var10001.nextBytes(forWrapping2);
            ParametersWithIV var9 = forWrapping3 = new ParametersWithIV;
            DESedeWrapEngine var10002 = this;
            KeyParameter var3 = this.param;
            var9.<init>(var3, var10002.iv);
            var8.paramPlusIV = forWrapping3;
         }
      } else if (param instanceof ParametersWithIV) {
         this.paramPlusIV = forWrapping3 = (ParametersWithIV)param;
         this.iv = forWrapping3.getIV();
         this.param = (KeyParameter)this.paramPlusIV.getParameters();
         if (!this.forWrapping) {
            throw new IllegalArgumentException("You should not supply an IV for unwrapping");
         }

         byte[] var4;
         if ((var4 = this.iv) == null || var4.length != 8) {
            throw new IllegalArgumentException("IV is not 8 octets");
         }
      }

   }

   public String getAlgorithmName() {
      return "DESede";
   }

   public byte[] wrap(byte[] in, int inOff, int inLen) {
      if (!this.forWrapping) {
         throw new IllegalStateException("Not initialized for wrapping");
      } else {
         System.arraycopy(in, inOff, in = new byte[inLen], 0, inLen);
         int var4;
         byte[] inOff1;
         int var10000 = var4 = inLen + (inOff1 = this.calculateCMSKeyChecksum(in)).length;
         byte[] var5;
         byte[] var10004 = var5 = new byte[var4];
         System.arraycopy(in, 0, var5, 0, inLen);
         int in1 = inOff1.length;
         System.arraycopy(inOff1, 0, var5, inLen, in1);
         in = new byte[var4];
         System.arraycopy(var10004, 0, in, 0, var4);
         inOff = var4 / this.engine.getBlockSize();
         if (var10000 % this.engine.getBlockSize() != 0) {
            throw new IllegalStateException("Not multiple of block length");
         } else {
            this.engine.init(true, this.paramPlusIV);

            int var12;
            for(inLen = 0; inLen < inOff; ++inLen) {
               var12 = inLen * this.engine.getBlockSize();
               this.engine.processBlock(in, var12, in, var12);
            }

            byte[] inLen1;
            var10000 = var12 = (inLen1 = this.iv).length + var4;
            byte[] var6 = new byte[var12];
            in1 = inLen1.length;
            System.arraycopy(inLen1, 0, var6, 0, in1);
            in1 = this.iv.length;
            System.arraycopy(in, 0, var6, in1, var4);
            byte[] var7 = new byte[var10000];

            for(in1 = 0; in1 < var12; var7[in1] = var6[var12 - ++in1]) {
            }

            ParametersWithIV in2;
            in2 = new ParametersWithIV.<init>(this.param, IV2);
            this.engine.init(true, in2);

            for(in1 = 0; in1 < inOff + 1; ++in1) {
               inLen = in1 * this.engine.getBlockSize();
               this.engine.processBlock(var7, inLen, var7, inLen);
            }

            return var7;
         }
      }
   }

   public byte[] unwrap(byte[] in, int inOff, int inLen) {
      if (this.forWrapping) {
         throw new IllegalStateException("Not set for unwrapping");
      } else if (in == null) {
         throw new InvalidCipherTextException("Null pointer as ciphertext");
      } else if (inLen % this.engine.getBlockSize() != 0) {
         throw new InvalidCipherTextException("Ciphertext not multiple of " + this.engine.getBlockSize());
      } else {
         ParametersWithIV in1;
         in1 = new ParametersWithIV.<init>(this.param, IV2);
         this.engine.init(false, in1);
         System.arraycopy(in, inOff, in = new byte[inLen], 0, inLen);

         for(inOff = 0; inOff < inLen / this.engine.getBlockSize(); ++inOff) {
            int var4 = inOff * this.engine.getBlockSize();
            this.engine.processBlock(in, var4, in, var4);
         }

         byte[] var12 = new byte[inLen];

         for(inOff = 0; inOff < inLen; var12[inOff] = in[inLen - ++inOff]) {
         }

         this.iv = in = new byte[8];
         int var5;
         byte[] inOff1;
         byte[] var10000 = inOff1 = new byte[var5 = inLen - 8];
         System.arraycopy(var12, 0, in, 0, 8);
         System.arraycopy(var12, 8, inOff1, 0, var5);
         ParametersWithIV var10004 = in1 = new ParametersWithIV;
         KeyParameter inOff2 = this.param;
         var10004.<init>(inOff2, this.iv);
         this.paramPlusIV = var10004;
         this.engine.init(false, in1);
         byte[] var6 = new byte[var5];
         System.arraycopy(var10000, 0, var6, 0, var5);

         for(int in2 = 0; in2 < var5 / this.engine.getBlockSize(); ++in2) {
            inOff = in2 * this.engine.getBlockSize();
            this.engine.processBlock(var6, inOff, var6, inOff);
         }

         int var7;
         in = new byte[var7 = var5 - 8];
         inOff1 = new byte[8];
         System.arraycopy(var6, 0, in, 0, var7);
         System.arraycopy(var6, var7, inOff1, 0, 8);
         if (this.checkCMSKeyChecksum(in, inOff1)) {
            return in;
         } else {
            throw new InvalidCipherTextException("Checksum inside ciphertext is corrupted");
         }
      }
   }
}
